function Create-ExMsalToken {
        $msalParams = @{
            ClientId     = '#TOREPLACE#'
            TenantId     = '#TOREPLACE#'
            ClientSecret = ('#TOREPLACE#' | ConvertTo-SecureString -AsPlainText -Force)
            Scopes   = "https://outlook.office.com/.default"
        }
        $script:TOKEN = Get-MsalToken @msalParams
        $TOKENTEST = $TOKEN
        #Write-Host "neuer Token - Expire $($token.ExpiresOn.LocalDatetime) Check $($(Get-Date).AddMinutes(-5))"
        Return 
}
function Get-ExMsalToken {
    if($TOKEN -ne "") {
        $DatePatternEX = "MM/dd/yyyy HH:mm:ss"
        if ($( Get-Date ) -gt [datetime]::ParseExact($token.ExpiresOn.LocalDatetime, $DatePatternEX, $null).AddMinutes(-5)) {
            $msalParams = @{
                ClientId     = '#TOREPLACE#'
                TenantId     = '#TOREPLACE#'
                ClientSecret = ('#TOREPLACE#' | ConvertTo-SecureString -AsPlainText -Force)
                Scopes   = "https://outlook.office.com/.default"
            }
            Write-Host "refresh Token - Expire $($token.ExpiresOn.LocalDatetime) Check $(Get-Date)"
            $script:TOKEN = Get-MsalToken @msalParams
            Return $TOKEN
        } else {
            Write-Host "Alter Token - Expire $($token.ExpiresOn.LocalDatetime) Check $(Get-Date)"
            Return $TOKEN
        }
    }else {
        $msalParams = @{
            ClientId     = '#TOREPLACE#'
            TenantId     = '#TOREPLACE#'
            ClientSecret = ('#TOREPLACE#' | ConvertTo-SecureString -AsPlainText -Force)
            Scopes   = "https://outlook.office.com/.default"
        }
        $script:TOKEN = Get-MsalToken @msalParams
        Write-Host "neuer Token - Expire $($token.ExpiresOn.LocalDatetime) Check $(Get-Date)"
        Return $TOKEN
    }
}
function New-CalendarItem {
    [CmdletBinding()]
    param(
        [Parameter(Position = 1, Mandatory = $true)]
        $Subject,
        [Parameter(Position = 2, Mandatory = $true)]
        $Body, 
        [Parameter(Position = 3, Mandatory = $true)]
        $Start, 
        [Parameter(Position = 4, Mandatory = $true)]
        $End, 
        [Parameter(Position = 5, Mandatory = $false)]
        $RequiredAttendees, 
        [Parameter(Position = 6, Mandatory = $false)]
        $OptionalAttendees, 
        [Parameter(Position = 7, Mandatory = $false)]
        $Location,
        [Parameter(Position = 8, Mandatory = $false)]
        $Impersonate,
        [Parameter(Position = 9, Mandatory = $false)]
        $AdminUser,
        [Parameter(Position = 10, Mandatory = $false)]
        $FreeBusyStatus = "Busy"
    )
    Begin {
        Write-Verbose "New-CalendarItem"		
    }
    Process {
        try {
            # Wenn der Server kein EWS / Exchange installiert hat, muss EwsManageApi.msi installiert werden....
            Add-Type -Path "$($ScriptDir)\modules\ews\Microsoft.Exchange.WebServices.dll"
            $Service = New-Object Microsoft.Exchange.WebServices.Data.ExchangeService -ArgumentList 'Exchange2013_SP1'
            $Service.Url = 'https://outlook.office365.com/EWS/Exchange.asmx'
            $Service.UseDefaultCredentials = $false
            $token = Get-ExMsalToken
            $Service.Credentials = [Microsoft.Exchange.WebServices.Data.OAuthCredentials]$token.AccessToken
            $service.Url = new-object Uri("https://outlook.office365.com/EWS/Exchange.asmx");
            $CleanGlobalObjectId = new-object Microsoft.Exchange.WebServices.Data.ExtendedPropertyDefinition([Microsoft.Exchange.WebServices.Data.DefaultExtendedPropertySet]::Meeting, 0x23, [Microsoft.Exchange.WebServices.Data.MapiPropertyType]::Binary);
            $psPropSet = new-object Microsoft.Exchange.WebServices.Data.PropertySet([Microsoft.Exchange.WebServices.Data.BasePropertySet]::FirstClassProperties);
            $psPropSet.Add($CleanGlobalObjectId);
        }catch {
            Write-Log -Message "Error: $($_.Exception.Message) - New-CalendarItem - Line Number: $($_.InvocationInfo.ScriptLineNumber)" -Level Error
            Return $false	
        }
        try {
            if ($Impersonate) {
                $ImpersonatedUserId = New-Object Microsoft.Exchange.WebServices.Data.ImpersonatedUserId -ArgumentList ([Microsoft.Exchange.WebServices.Data.ConnectingIdType]::SmtpAddress), $Impersonate 
                $service.ImpersonatedUserId = $ImpersonatedUserId   
            }

            $appointment = New-Object Microsoft.Exchange.WebServices.Data.Appointment -ArgumentList $service
            $appointment.Subject = $Subject
            $appointment.Body = $Body
            $appointment.LegacyFreeBusyStatus = $FreeBusyStatus
            $appointment.Categories.Add('Personio')

            $appointment.Start = $Start
            $appointment.End = $End
            if ($RequiredAttendees) {
                $RequiredAttendees | % { [void]$appointment.RequiredAttendees.Add($_) }
            }
            if ($OptionalAttendees) {
                $OptionalAttendees | % { [void]$appointment.RequiredAttendees.Add($_) }
            }
            if ($Location) {
                $appointment.Location = $Location
            }
            ######################################################################################
            ######################################################################################
            # Schreibe Eindeutige ID im Kalendereintrag als Extendet Property 
            # und übergebe dies am Script
            ######################################################################################
            ######################################################################################
            echo $appointment
            $appointment.Save([Microsoft.Exchange.WebServices.Data.SendInvitationsMode]::SendToAllAndSaveCopy) | out-null
          
            $appointment.Load($psPropSet); 
            $CalIdVal = $null;
            $tmpvar = $appointment.TryGetProperty($CleanGlobalObjectId, [ref]$CalIdVal); 
            $CalIdVal64 = [Convert]::ToBase64String($CalIdVal) | Select -Last 1
            Return "$($CalIdVal64)"
        }
        catch {
            Write-Log -Message "Error: $($_.Exception.Message) - New-CalendarItem - Line Number: $($_.InvocationInfo.ScriptLineNumber)" -Level Error
            Return $false	
        }
    }	 
    End {
        Write-Verbose "Stop SET-UserVacationHelperTable"
    }  
}

function Send-ErrorMail {
    [CmdletBinding()]
    param(
        [Parameter(Position = 1, Mandatory = $true)]
        $Mail,
        [Parameter(Position = 2, Mandatory = $true)]
        $VACSTART, 
        [Parameter(Position = 3, Mandatory = $true)]
        $VACENDE, 
        [Parameter(Position = 4, Mandatory = $true)]
        $VACID
    )
    Begin {
        Write-Verbose "Send-ErrorMail"		
    }
    Process {
        try {
            $server = $ini.email.server
            $from = $ini.email.from
            $to = $ini.email.to
            $subject = "Fehler bei der EWS-Schnittstelle - Personio"
            $body = "Es ist ein Fehler passiert,<br><br>
               
                    <table>
                    <tr>
                        <th>Bezeichnung</th>
                        <th>Wert</th> 
                    </tr>
                    <tr>
                        <td>Benutzer</td>
                        <td>$($Mail)</td>
                    </tr>
                    <tr>
                        <td>Urlaubsbeginn</td>
                        <td>$($VACSTART)</td>
                    </tr>
                    <tr>
                        <td>Urlaubsende</td>
                        <td>$($VACENDE)</td>
                    </tr>
                    <tr>
                        <td>UrlaubsID</td>
                        <td>$($VACID)</td>
                    </tr>
                    </table><br><br>
                   
                    Bitte prüfen und korrigieren"
            send-mailmessage -from $from -to $to -subject $subject -body $body -BodyAsHtml -smtpServer $server -Port 26
            Return $true
        }	
        catch {
            Write-Log -Message "Error: $($_.Exception.Message) - Send-ErrorMail - Line Number: $($_.InvocationInfo.ScriptLineNumber)" -Level Error
            Return $false	
        }
    }	 
    End {
        Write-Verbose "Stop Send-ErrorMail"
    }  
}

function Remove-CalendarItem {
    [CmdletBinding()]
    param(
        [Parameter(Position = 1, Mandatory = $true)]
        $CALID,
        [Parameter(Position = 2, Mandatory = $true)]
        $Impersonate
    )
    Begin {
        Write-Verbose "Remove-CalendarItem"		
    }
    Process {
        try {
            # Wenn der Server kein EWS / Exchange installiert hat, muss EwsManageApi.msi installiert werden....
            Add-Type -Path "$($ScriptDir)\modules\ews\Microsoft.Exchange.WebServices.dll"
            $Service = New-Object Microsoft.Exchange.WebServices.Data.ExchangeService -ArgumentList 'Exchange2013_SP1'
            $Service.Url = 'https://outlook.office365.com/EWS/Exchange.asmx'
            $Service.UseDefaultCredentials = $false
            $token = Get-ExMsalToken
            $Service.Credentials = [Microsoft.Exchange.WebServices.Data.OAuthCredentials]$token.AccessToken
        }catch {
            Write-Log -Message "Error: $($_.Exception.Message) - New-CalendarItem - Line Number: $($_.InvocationInfo.ScriptLineNumber)" -Level Error
            Return $false	
        }
        try {
            if ($Impersonate) {
                $ImpersonatedUserId = New-Object Microsoft.Exchange.WebServices.Data.ImpersonatedUserId -ArgumentList ([Microsoft.Exchange.WebServices.Data.ConnectingIdType]::SmtpAddress), $Impersonate 
                $service.ImpersonatedUserId = $ImpersonatedUserId   
            }
            $folderid = new-object Microsoft.Exchange.WebServices.Data.FolderId([Microsoft.Exchange.WebServices.Data.WellKnownFolderName]::Calendar, "$($Impersonate)")
            $Calendar = [Microsoft.Exchange.WebServices.Data.Folder]::Bind($service, $folderid)  
            Write-Host "CAL: $($CALID.Replace("Microsoft.Exchange.WebServices.Data.Appointment ",''))"
            $CalIdVal64 = "$($CALID.Replace("Microsoft.Exchange.WebServices.Data.Appointment ",''))"
            $CleanGlobalObjectId = new-object Microsoft.Exchange.WebServices.Data.ExtendedPropertyDefinition([Microsoft.Exchange.WebServices.Data.DefaultExtendedPropertySet]::Meeting, 0x23, [Microsoft.Exchange.WebServices.Data.MapiPropertyType]::Binary);
            $ivItemView = New-Object Microsoft.Exchange.WebServices.Data.ItemView(1) 
            $sfSearchFilter = new-object Microsoft.Exchange.WebServices.Data.SearchFilter+IsEqualTo($CleanGlobalObjectId, $CalIdVal64);
            $fiResult = $Calendar.FindItems($sfSearchFilter, $ivItemView) 
            if ( ($fiResult.Items[0]).Categories -eq "Personio") {
                $fiResult.Items[0].Delete([Microsoft.Exchange.WebServices.Data.DeleteMode]::SoftDelete)
            }
            Return $true
        }	
        catch {
            Write-Log -Message "Error: $($_.Exception.Message) - Line Number: $($_.InvocationInfo.ScriptLineNumber) - CALID: $([string]$CALID)" -Level Error
            Return $false	
        }
    }	 
    End {
        Write-Verbose "Stop Remove-CalendarItem"
    }  
}

function Remove-CalendarItemByDate {
    [CmdletBinding()]
    param(
        [Parameter(Position = 1, Mandatory = $true)]
        $Start,
        [Parameter(Position = 2, Mandatory = $true)]
        $End,
        [Parameter(Position = 3, Mandatory = $true)]
        $Impersonate
    )
    Begin {
        Write-Verbose "Remove-CalendarItem"		
    }
    Process {
        try {
            Add-Type -Path "$($ScriptDir)\modules\ews\Microsoft.Exchange.WebServices.dll"
            $Service = New-Object Microsoft.Exchange.WebServices.Data.ExchangeService -ArgumentList 'Exchange2013_SP1'
            $Service.Url = 'https://outlook.office365.com/EWS/Exchange.asmx'
            $Service.UseDefaultCredentials = $false
            $token = Get-ExMsalToken
            $Service.Credentials = [Microsoft.Exchange.WebServices.Data.OAuthCredentials]$token.AccessToken
            $service.Url = new-object Uri("https://outlook.office365.com/EWS/Exchange.asmx");
        } 
        catch {
            Write-Log -Message "Error: $($_.Exception.Message) - Line Number: $($_.InvocationInfo.ScriptLineNumber)" -Level Error
            Return $false 
        }
        try {

            if ($Impersonate) {
                $ImpersonatedUserId = New-Object Microsoft.Exchange.WebServices.Data.ImpersonatedUserId -ArgumentList ([Microsoft.Exchange.WebServices.Data.ConnectingIdType]::SmtpAddress), $Impersonate 
                $service.ImpersonatedUserId = $ImpersonatedUserId   
            }

            $folderid = new-object Microsoft.Exchange.WebServices.Data.FolderId([Microsoft.Exchange.WebServices.Data.WellKnownFolderName]::Calendar, "$($Impersonate)")
            $Calendar = [Microsoft.Exchange.WebServices.Data.Folder]::Bind($service, $folderid)  

            $CalendarView = New-Object Microsoft.Exchange.WebServices.Data.CalendarView($Start, $End, 1000)    
            $fiItems = $service.FindAppointments($Calendar.Id, $CalendarView)    

            foreach ($Item in $fiItems.Items) {     
                if ($item.Categories -ccontains "Personio") {
                    "Start    : " + $Item.Start  
                    "Subject     : " + $Item.Subject  
                    $Item.Delete([Microsoft.Exchange.WebServices.Data.DeleteMode]::SoftDelete)
                } 
            }  
            Return $true
        }	
        catch {
            Write-Log -Message "Error: $($_.Exception.Message) - Line Number: $($_.InvocationInfo.ScriptLineNumber)" -Level Error
            Return $false	
        }
    }	 
    End {
        Write-Verbose "Stop Remove-CalendarItem"
    }  
}


function Remove-DupCalendarItem {
    [CmdletBinding()]
    param(
        [Parameter(Position = 1, Mandatory = $true)]
        $START,
        [Parameter(Position = 2, Mandatory = $true)]
        $END,
        [Parameter(Position = 3, Mandatory = $true)]
        $Impersonate,
        [Parameter(Position = 3, Mandatory = $true)]
        $USERID
        
    )
    Begin {
        Write-Verbose "Remove-DupCalendarItem"		
    }
    Process {
        try {
            # Wenn der Server kein EWS / Exchange installiert hat, muss EwsManageApi.msi installiert werden....
            Add-Type -Path "$($ScriptDir)\modules\ews\Microsoft.Exchange.WebServices.dll"
            $Service = New-Object Microsoft.Exchange.WebServices.Data.ExchangeService -ArgumentList 'Exchange2013_SP1'
            $Service.Url = 'https://outlook.office365.com/EWS/Exchange.asmx'
            $Service.UseDefaultCredentials = $false
            $token = Get-ExMsalToken
            $Service.Credentials = [Microsoft.Exchange.WebServices.Data.OAuthCredentials]$token.AccessToken
            $service.Url = new-object Uri("https://outlook.office365.com/EWS/Exchange.asmx");

        }
        catch {
            Write-Log -Message "Error: $($_.Exception.Message) - Line Number: $($_.InvocationInfo.ScriptLineNumber)" -Level Error
            Return $false	
        }
        try {

            if ($Impersonate) {
                $ImpersonatedUserId = New-Object Microsoft.Exchange.WebServices.Data.ImpersonatedUserId -ArgumentList ([Microsoft.Exchange.WebServices.Data.ConnectingIdType]::SmtpAddress), $Impersonate 
                $service.ImpersonatedUserId = $ImpersonatedUserId   
            }
            $folderid = new-object Microsoft.Exchange.WebServices.Data.FolderId([Microsoft.Exchange.WebServices.Data.WellKnownFolderName]::Calendar, "$($Impersonate)")
            $Calendar = [Microsoft.Exchange.WebServices.Data.Folder]::Bind($service, $folderid)  
            $CalendarView = New-Object Microsoft.Exchange.WebServices.Data.CalendarView($START, $END , 1000)    
            $fiItems = $service.FindAppointments($Calendar.Id, $CalendarView)    
            if ($($fiItems | where { $_.Categories -ccontains "Personio" -and $_.Start -eq $START -and $_.End -eq "$END" }).Count -gt 1) {
                Write-Log -Message "Kalendereintrag: $($($START)) bis $($($END)) für $($USERID) Anzahl $($($fiItems | where { $_.Categories -ccontains "Personio" -and $_.Start -eq $START -and $_.End -eq "$END" }).Count) mehrfach vorhanden... delete wird anhand Datum durchgeführt" -Level WARN
                foreach ($Item in $($fiItems.items | where { $_.Categories -ccontains "Personio" -and $_.Start -eq $START -and $_.End -eq "$END" })) {     
                    $CalIdVal = $null;
                    $psPropSet = new-object Microsoft.Exchange.WebServices.Data.PropertySet([Microsoft.Exchange.WebServices.Data.BasePropertySet]::FirstClassProperties);
                    $CleanGlobalObjectId = new-object Microsoft.Exchange.WebServices.Data.ExtendedPropertyDefinition([Microsoft.Exchange.WebServices.Data.DefaultExtendedPropertySet]::Meeting, 0x23, [Microsoft.Exchange.WebServices.Data.MapiPropertyType]::Binary);
                    $psPropSet.Add($CleanGlobalObjectId);
                    $ItemLoad = $item.Load($psPropSet); 
                    $GetProp = $item.TryGetProperty($CleanGlobalObjectId, [ref]$CalIdVal); 
                    $EXCHANGEID = [Convert]::ToBase64String($CalIdVal) | Select -Last 1
                    $EXPERID = $(GET-SQLITEByExID -USERID "$($USERID)" -EXID $EXCHANGEID).Id
                    if (  $EXPERID -eq $null ) {
                        $Item.Delete([Microsoft.Exchange.WebServices.Data.DeleteMode]::SoftDelete)
                        Write-Log -Message "Kalendereintrag: $($($Item.Start)) bis $($($Item.End)) für $($USERID) - $($EXCHANGEID) mehrfach vorhanden... delete wird anhand Datum durchgeführt" -Level WARN
                    }
                }  
            }
        }	
        catch {
            Write-Log -Message "Error: $($_.Exception.Message) - Line Number: $($_.InvocationInfo.ScriptLineNumber)" -Level Error
            Return $false	
        }
    }	 
    End {
        Write-Verbose "Stop Remove-DupCalendarItem"
    }  
}
