[CmdletBinding()] 
param ( 
    
    [Parameter(Mandatory=$true,ValueFromPipeline=$true)] 
    [ValidateScript({ Test-Path -PathType:Leaf -Path:$_ })] 
    [string] $IniFile 
) 
 
begin { 
    Write-Verbose "$($MyInvocation.Line)" 
    $iniObj = @{} 
} 
 
process { 
    switch -regex -File $IniFile { 
        "^\[(.+)\]$" { 
            $section = $matches[1] 
            $iniObj[$section] = @{} 
        } 
        "(?<key>^[^\#\;\=]*)[=?](?<value>.+)" { 
            $key    = $matches.key.Trim() 
            $value    = $matches.value.Trim() 
            if ( ($value -like '$(*)') -or ($value -like '"*"') ) { 
                $value = Invoke-Expression $value 
            }  
            if ( $section ) { 
                $iniObj[$section][$key] = $value 
            } else { 
                $iniObj[$key] = $value 
            } 
        } 
    } 
} 
 
end { 
    return $iniObj 
} 